#ifndef Zeit_H
#define Zeit_H

#include <time.h>

#include "Betriebssystem.h"
#include "ReferenceCounter.h"

namespace Framework
{
    class Text;       //! Text.h
    class Uhrzeit;    //! Aus dieser Datei
    class Datum;      //! Aus dieser Datei
    class Zeit;       //! Aus dieser Datei
    class ZeitMesser; //! Aus dieser Datei

    //! Diese Klasse speichert eine Uhrzeit in form von Stunden, Minuten und
    //! Sekunden
    class Uhrzeit : public virtual ReferenceCounter
    {
    private:
        __int64 stunde, minute, sekunde;
        int update();

    public:
        // Erzeugt ein neues Uhrzeit Zeichnung mit den Standartwerten 00:00:00
        DLLEXPORT Uhrzeit();

        // Setzt die Uhrzeit durch kopieren.
        //! \param zeit Aus diesem Zeichnung werden die Werte fr Stunde, Minute
        //! und Sekunde kopiert.
        DLLEXPORT int setUhrzeit(Uhrzeit* zeit);
        // Setzt die Uhrzeit.
        //! \param stunde Die Stunde der neuen Uhrzeit.
        //! \param minute Die Minute der neuen Uhrzeit.
        //! \param sekunde Die Sekunde der neuen Uhrzeit.
        //! \return Die Anzahl briger Tage.
        //! Beispiel: setUhrzeit( 30, 30, 30 ); return: 1, gespeichert: 6:30:30
        DLLEXPORT int setUhrzeit(int stunde, int minute, int sekunde);
        // Setzt die Uhrzeit.
        //! \param format Eine Zeichenfolge, die bestimmt in welcher Form die
        //! Uhrzeit in (zeit) vorliegt. h=Stunde, i=Minute,s=Sekunde. \param
        //! zeit Eine Zeichenfolge die die neue Uhrzeit enthlt. \return Die
        //! Anzahl briger Tage. Beispiel: setUhrzeit( "h:i:s", "05:30:00" );
        //! Beispiel: setUhrzeit( "h:i:s", "30:30:00" ); return: 1, gespeichert:
        //! 6:30:00
        DLLEXPORT int setUhrzeit(const char* format, const char* zeit);
        // Setzt die Uhrzeit.
        //! \param format Eine Zeichenfolge, die bestimmt in welcher Form die
        //! Uhrzeit in (zeit) vorliegt. h=Stunde, i=Minute,s=Sekunde. \param
        //! zeit Ein Text Zeichnung, dass die neue Uhrzeit enthlt. \return Die
        //! Anzahl briger Tage. Beispiel: setUhrzeit( "h:i:s", new Text(
        //! "05:30:00" ) ); Beispiel: setUhrzeit( "h:i:s", new Text( "30:30:00"
        //! ) ); return: 1, gespeichert: 6:30:00
        DLLEXPORT int setUhrzeit(const char* format, Text* zeit);
        //! setzt die Stunde.
        //! \param stunde Die Stunde, die gespeichert werden soll.
        //! \return Die Anzahl briger Tage.
        //! Beispiel: setStunde( 30 ); return: 1 Tag, gespeicherte Stunden: 6
        DLLEXPORT int setStunde(int stunde);
        //! setzt die Minute.
        //! \param minute Die Minute, die gespeichert werden soll.
        //! \return Die Anzahl briger Tage.
        //! Beispiel: Alte Zeit: 23:50:10, setMinute( 80 ); return: 1,
        //! gespeicherte Zeit: 01:10:10
        DLLEXPORT int setMinute(int minute);
        //! setzt die Sekunde.
        //! \param minute Die Sekunde, die gespeichert werden soll.
        //! \return Die Anzahl briger Tage.
        //! Beispiel: Alte Zeit: 23:59:10, setSekunde( 80 ); return: 1,
        //! gespeicherte Zeit: 00:00:30
        DLLEXPORT int setSekunde(int sekunde);
        //! errechnet die Summe von dieser und einer anderen Uhrzeit und
        //! speichert diese. \param zeit Die Uhrzeit, deren Werte addiert werden
        //! sollen. \return Die Anzahl briger Tage. Beispiel: Alte Zeit:
        //! 19:40:18, plusZeit( (10:05:30) ); return 1, gespeicherte Zeit:
        //! 05:45:48
        DLLEXPORT int plusUhrzeit(Uhrzeit* zeit);
        //! errechnet die Summe von dieser und einer anderen Uhrzeit und
        //! speichert diese. \param stunde Die Stunden die Werte addiert werden
        //! sollen. \param minute Die Minute die Werte addiert werden sollen.
        //! \param sekunde Die Sekunde die Werte addiert werden sollen.
        //! \return Die Anzahl briger Tage.
        //! Beispiel: Alte Zeit: 19:40:18, plusZeit( 10, 5, 30 ); return 1,
        //! gespeicherte Zeit: 05:45:48
        DLLEXPORT int plusUhrzeit(int stunde, int minute, int sekunde);
        //! errechnet die Summe von dieser und einer anderen Uhrzeit und
        //! speichert diese. \param format Eine Zeichenfolge, die bestimmt in
        //! welcher Form die Uhrzeit in (zeit) vorliegt. h=Stunde,
        //! i=Minute,s=Sekunde. \param zeit Eine Zeichenfolge die die neue
        //! Uhrzeit enthlt. \return Die Anzahl briger Tage. Beispiel:
        //! plusUhrzeit( "h:i:s", "05:20:00" ); Beispiel: Alte Zeit: 19:40:18,
        //! plusZeit( "h:i:s", "10:05:30" ); return 1, gespeicherte Zeit:
        //! 05:45:48
        DLLEXPORT int plusUhrzeit(const char* format, const char* zeit);
        //! errechnet die Summe von dieser und einer anderen Uhrzeit und
        //! speichert diese. \param format Eine Zeichenfolge, die bestimmt in
        //! welcher Form die Uhrzeit in (zeit) vorliegt. h=Stunde,
        //! i=Minute,s=Sekunde. \param zeit Ein Text Zeichnung, welches die neue
        //! Uhrzeit enthlt. \return Die Anzahl briger Tage. Beispiel:
        //! plusUhrzeit( "h:i:s", new Text( "05:20:00" ) ); Beispiel: Alte Zeit:
        //! 19:40:18, plusZeit( "h:i:s", new Text( "10:05:30" ) ); return 1,
        //! gespeicherte Zeit: 05:45:48
        DLLEXPORT int plusUhrzeit(const char* format, Text* zeit);
        //! errechnet die Summe von dieser und einer anderen Uhrzeit und
        //! speichert diese. \param stunde die draufzurechnenden Stunden.
        //! \return Die Anzahl briger Tage.
        //! Beispiel: Alte Zeit: 20:50:30, plusStunde( 10 ); return 1,
        //! gespeicherte Zeit: 6:50:30
        DLLEXPORT int plusStunde(__int64 stunde);
        //! errechnet die Summe von dieser und einer anderen Uhrzeit und
        //! speichert diese. \param minute die draufzurechnenden Minuten.
        //! \return Die Anzahl briger Tage.
        //! Beispiel: Alte Zeit: 23:50:30, plusMinute( 11 ); return 1,
        //! gespeicherte Zeit: 00:01:30
        DLLEXPORT int plusMinute(__int64 minute);
        //! errechnet die Summe von dieser und einer anderen Uhrzeit und
        //! speichert diese. \param sekunde die draufzurechnenden Sekunden.
        //! \return Die Anzahl briger Tage.
        //! Beispiel: Alte Zeit: 23:59:30, plusSekunde( 40 ); return 1,
        //! gespeicherte Zeit: 00:00:10
        DLLEXPORT int plusSekunde(__int64 sekunde);
        //! zieht eine gegebene Uhrzeit von dieser ab und speichert das
        //! Ergebnis. \param zeit Die Uhrzeit, die von dieser abgezogen werden
        //! soll. \return Die Anzahl briger Tage. Beispiel Alte Zeit: 10:40:18,
        //! minusUhrzeit( (19:05:30) ); return -1, gespeicherte Zeit: 15:34:48
        DLLEXPORT int minusUhrzeit(Uhrzeit* zeit);
        //! zieht eine gegebene Uhrzeit von dieser ab und speichert das
        //! Ergebnis. \param stunde Die Stunden, die von dieser Uhrzeit
        //! abgezogen werden sollen. \param minute Die Minuten, die von dieser
        //! Uhrzeit abgezogen werden sollen. \param sekunde Die Sekunden, die
        //! von dieser Uhrzeit abgezogen werden sollen. \return Die Anzahl
        //! briger Tage. Beispiel Alte Zeit: 10:40:18, minusUhrzeit( 19, 05, 30
        //! ); return -1, gespeicherte Zeit: 15:34:48
        DLLEXPORT int minusUhrzeit(int stunde, int minute, int sekunde);
        //! zieht eine gegebene Uhrzeit von dieser ab und speichert das
        //! Ergebnis. \param format Eine Zeichenfolge, die bestimmt in welcher
        //! Form die Uhrzeit in (zeit) vorliegt. h=Stunde, i=Minute,s=Sekunde.
        //! \param zeit Eine Zeichenfolge die die neue Uhrzeit enthlt.
        //! \return Die Anzahl briger Tage.
        //! Beispiel Alte Zeit: 10:40:18, minusUhrzeit( "h:i:s", "19:05:30" );
        //! return -1, gespeicherte Zeit: 15:34:48
        DLLEXPORT int minusUhrzeit(const char* format, const char* zeit);
        //! zieht eine gegebene Uhrzeit von dieser ab und speichert das
        //! Ergebnis. \param format Eine Zeichenfolge, die bestimmt in welcher
        //! Form die Uhrzeit in (zeit) vorliegt. h=Stunde, i=Minute,s=Sekunde.
        //! \param zeit Ein Text Zeichnung, welches die neue Uhrzeit enthlt.
        //! \return Die Anzahl briger Tage.
        //! Beispiel Alte Zeit: 10:40:18, minusUhrzeit( "h:i:s", new Text(
        //! "19:05:30" ) ); return -1, gespeicherte Zeit: 15:34:48
        DLLEXPORT int minusUhrzeit(const char* format, Text* zeit);
        //! zieht eine gegebene Zeit von dieser ab und speichert das Ergebnis.
        //! \param stunde Die abzuziehenden Stunden.
        //! \return Die Anzahl briger Tage.
        //! Beispiel Alte Zeit: 10:40:18, minusStunde( 19 ); return -1,
        //! gespeicherte Zeit: 15:40:18
        DLLEXPORT int minusStunde(__int64 stunde);
        //! zieht eine gegebene Zeit von dieser ab und speichert das Ergebnis.
        //! \param minute Die abzuziehenden Minuten.
        //! \return Die Anzahl briger Tage.
        //! Beispiel Alte Zeit: 00:40:18, minusStunde( 50 ); return -1,
        //! gespeicherte Zeit: 23:50:18
        DLLEXPORT int minusMinute(__int64 minute);
        //! zieht eine gegebene Zeit von dieser ab und speichert das Ergebnis.
        //! \param sekunde Die abzuziehenden Sekunden.
        //! \return Die Anzahl briger Tage.
        //! Beispiel Alte Zeit: 00:00:20, minusStunde( 50 ); return -1,
        //! gespeicherte Zeit: 23:59:30
        DLLEXPORT int minusSekunde(__int64 sekunde);

        //! gibt die Stunde zurck.
        DLLEXPORT int getStunde() const;
        //! gibt die Minute zurck.
        DLLEXPORT int getMinute() const;
        //! gibt die Sekunde zurck.
        DLLEXPORT int getSekunde() const;
        //! gibt die Uhrzeit als Text formatiert zurck.
        //! \param format Eine Zeichenfolge, die bestimmt in welcher Form die
        //! Uhrzeit zurckgegeben werden soll. h=Stunde, i=Minute,s=Sekunde.
        //! Beispiel: Zeit: 5:40:39, getUhrzeit( "h:i:s" ); return: "5:40:39"
        DLLEXPORT Text* getUhrzeit(const char* format) const;
        //! prft, ob die Uhrzeit gleich zeit ist.
        //! \param zeit Die Uhrzeit, mit der diese verglichen werden soll.
        //! \return (true), wenn die beiden Zeiten geich sind. (false), wenn die
        //! Zeiten nicht gleich sind.
        DLLEXPORT bool istGleich(Uhrzeit* zeit) const;
        //! prft, ob die Uhrzeit gleich zeit ist.
        //! \param format Eine Zeichenfolge, die bestimmt in welcher Form die
        //! Uhrzeit in (zeit) vorliegt. h=Stunde, i=Minute,s=Sekunde. \param
        //! zeit Eine Zeichenfolge die die andere Uhrzeit enthlt. \return
        //! (true), wenn die beiden Zeiten geich sind. (false), wenn die Zeiten
        //! nicht gleich sind.
        DLLEXPORT bool istGleich(const char* format, const char* zeit) const;
        //! prft, ob die Uhrzeit gleich zeit ist.
        //! \param format Eine Zeichenfolge, die bestimmt in welcher Form die
        //! Uhrzeit in (zeit) vorliegt. h=Stunde, i=Minute,s=Sekunde. \param
        //! zeit Ein Text Zeichnung, welches die andere Uhrzeit enthlt. \return
        //! (true), wenn die beiden Zeiten geich sind. (false), wenn die Zeiten
        //! nicht gleich sind.
        DLLEXPORT bool istGleich(const char* format, Text* zeit) const;
        //! prft, ob die Uhrzeit gleich der bergebenen Zeit ist. Hier wird
        //! nicht beachtet, ob die gegebene Zeit eine gltige Uhrzeit ist.
        //! \param stunde Die Stunden der zu berprfenden Uhrzeit
        //! \param minute Die Minuten der zu berprfenden Uhrzeit
        //! \param sekunde Die Sekunden der zu berprfenden Uhrzeit
        //! \return (true), wenn die beiden Zeiten geich sind. (false), wenn die
        //! Zeiten nicht gleich sind.
        DLLEXPORT bool istGleich(int stunde, int minute, int sekunde) const;
        //! prft, ob die Stunde gleich der gegebenen Stunde ist.
        //! \param stunde die zu prfende Stunde.
        //! \return (true), wenn die Stunden gleich sind. (false), wenn sie
        //! nicht gleich sind.
        DLLEXPORT bool stundeGleich(int stunde) const;
        //! prft, ob die Minute gleich der gegebenen Minute ist.
        //! \param minute die zu prfende Minute.
        //! \return (true), wenn die Minuten gleich sind. (false), wenn sie
        //! nicht gleich sind.
        DLLEXPORT bool minuteGleich(int minute) const;
        //! prft, ob die Sekunde gleich der gegebenen Sekunde ist.
        //! \param sekunde die zu prfende Sekunde.
        //! \return (true), wenn die Sekunden gleich sind. (false), wenn sie
        //! nicht gleich sind.
        DLLEXPORT bool sekundeGleich(int sekunde) const;
        //! prft, ob die Uhrzeit kleiner als zeit ist.
        //! \param zeit Die zu prfende Uhrzeit.
        //! \return (true), wenn die gespeicherte Zeit kleiner als die
        //! bergebene Zeit ist. (false) sonst. Beispiel: (5:30:00).istKleiner(
        //! (10:40:29) ); return true
        DLLEXPORT bool istKleiner(Uhrzeit* zeit) const;
        //! prft, ob die Uhrzeit kleiner als die bergebene Zeit ist.
        //! \param stunde Die Stunden der zu prfenden Zeit.
        //! \param minute Die Minute der zu prfenden Zeit.
        //! \param sekunde Die Sekunde der zu prfenden Zeit.
        //! \return (true), wenn die gespeicherte Zeit kleiner als die
        //! bergebene Zeit ist. (false) sonst. Beispiel: (5:30:00).istKleiner(
        //! 10, 40, 29 ); return true
        DLLEXPORT bool istKleiner(int stunde, int minute, int sekunde) const;
        //! prft, ob die Uhrzeit kleiner als die bergebene Zeit ist.
        //! \param format Eine Zeichenfolge, die bestimmt in welcher Form die
        //! Uhrzeit in (zeit) vorliegt. h=Stunde, i=Minute,s=Sekunde. \param
        //! zeit Eine Zeichenfolge die die andere Uhrzeit enthlt. \return
        //! (true), wenn die gespeicherte Zeit kleiner als die bergebene Zeit
        //! ist. (false) sonst. Beispiel: (5:30:00).istKleiner( "h:i:s",
        //! "10:40:29" ); return true
        DLLEXPORT bool istKleiner(const char* format, const char* zeit) const;
        //! prft, ob die Uhrzeit kleiner als die bergebene Zeit ist.
        //! \param format Eine Zeichenfolge, die bestimmt in welcher Form die
        //! Uhrzeit in (zeit) vorliegt. h=Stunde, i=Minute,s=Sekunde. \param
        //! zeit Ein Text Zeichnung, welches die andere Uhrzeit enthlt. \return
        //! (true), wenn die gespeicherte Zeit kleiner als die bergebene Zeit
        //! ist. (false) sonst. Beispiel: (5:30:00).istKleiner( "h:i:s",
        //! "10:40:29" ); return true
        DLLEXPORT bool istKleiner(const char* format, Text* zeit) const;
        //! prft, ob die Uhrzeit grer als zeit ist.
        //! \param zeit Die zu prfende Uhrzeit.
        //! \return (true), wenn die gespeicherte Zeit grer als die bergebene
        //! Zeit ist. (false) sonst. Beispiel: (5:30:00).istGrer( (10:40:29)
        //! ); return false
        DLLEXPORT bool istLater(Uhrzeit* zeit) const;
        //! prft, ob die Uhrzeit grer als die bergebene Zeit ist.
        //! \param stunde Die Stunden der zu prfenden Zeit.
        //! \param minute Die Minute der zu prfenden Zeit.
        //! \param sekunde Die Sekunde der zu prfenden Zeit.
        //! \return (true), wenn die gespeicherte Zeit grer als die bergebene
        //! Zeit ist. (false) sonst. Beispiel: (5:30:00).istGrer( 10, 40, 29
        //! ); return false
        DLLEXPORT bool istLater(int stunde, int minute, int sekunde) const;
        //! prft, ob die Uhrzeit grer als die bergebene Zeit ist.
        //! \param format Eine Zeichenfolge, die bestimmt in welcher Form die
        //! Uhrzeit in (zeit) vorliegt. h=Stunde, i=Minute,s=Sekunde. \param
        //! zeit Eine Zeichenfolge die die andere Uhrzeit enthlt. \return
        //! (true), wenn die gespeicherte Zeit grer als die bergebene Zeit
        //! ist. (false) sonst. Beispiel: (5:30:00).istGrer( "h:i:s",
        //! "10:40:29" ); return false
        DLLEXPORT bool istLater(const char* format, const char* zeit) const;
        //! prft, ob die Uhrzeit grer als die bergebene Zeit ist.
        //! \param format Eine Zeichenfolge, die bestimmt in welcher Form die
        //! Uhrzeit in (zeit) vorliegt. h=Stunde, i=Minute,s=Sekunde. \param
        //! zeit Ein Text Zeichnung, welches die andere Uhrzeit enthlt. \return
        //! (true), wenn die gespeicherte Zeit grer als die bergebene Zeit
        //! ist. (false) sonst. Beispiel: (5:30:00).istGrer( "h:i:s",
        //! "10:40:29" ); return false
        DLLEXPORT bool istLater(const char* format, Text* zeit) const;
    };

    //! Diese Klasse speichert ein Datum in Form von Jahr, Monat und Tagen
    class Datum : public virtual ReferenceCounter
    {
    private:
        int jahr, monat, tag;
        int* maxTage;
        void update();

    public:
        //! Erzeugt ein neues Datum Zeichnung mit den Standartwerten 0.0.0.
        DLLEXPORT Datum();
        //! Lscht das Datum.
        DLLEXPORT ~Datum();

        //! Setzt das Datum durch kopieren.
        //! \param datum Das zu speichernde Datum.
        DLLEXPORT void setDatum(Datum* datum);
        //! Setzt das Datum zu den bergebenen Werten.
        //! \param jahr Das Jahr des neuen Datums.
        //! \param monat Der Monat des neuen Datums.
        //! \param tag Der Tag des neuen Datums.
        DLLEXPORT void setDatum(int jahr, int monat, int tag);
        //! Setzt das Datum zu dem bergebenen Wert.
        //! \param format Eine Zeichenkette, die angibt in welcher Form das
        //! Datum in (datum) vorhanden ist. y=Jahr, m=Monat, d=Tag. \param datum
        //! Eine Zeichenkette, die das neue Datum enthlt. Beispiel: setDatum(
        //! "y-m-d", "2016-01-25" );
        DLLEXPORT void setDatum(const char* format, const char* datum);
        //! Setzt das Datum zu dem bergebenen Wert.
        //! \param format Eine Zeichenkette, die angibt in welcher Form das
        //! Datum in (datum) vorhanden ist. y=Jahr, m=Monat, d=Tag. \param datum
        //! Ein Text Zeichnung, welches das neue Datum enthlt. Beispiel:
        //! setDatum( "y-m-d", new Text(  "2016-01-25" ) );
        DLLEXPORT void setDatum(const char* format, Text* datum);
        //! ndert das Jahr des Datums.
        //! \param jahr Das neue Jahr.
        DLLEXPORT void setJahr(int jahr);
        //! ndert den Monat des Datums.
        //! \param monat Der neue Monat.
        DLLEXPORT void setMonat(int monat);
        //! ndert den Tag des Datums.
        //! \param tag Der neue Tag.
        DLLEXPORT void setTag(int tag);
        //! Addiert zum aktuellen Datum das bergebene Datum dazu und speichert
        //! das Ergebnis. \param datum Das zu addierende Datum. Beispiel:
        //! ( 1.11.1995 ).plusDatum( ( 5.2.7 ) ); neues Datum: 6.1.2003
        DLLEXPORT void plusDatum(Datum* datum);
        //! Addiert zum aktuellen Datum das bergebene Datum dazu und speichert
        //! das Ergebnis. \param jahr Das zu addierende Jahr. \param monat Der
        //! zu addierende Monat. \param tag Der zu addierende Tag. Beispiel:
        //! ( 1.11.1995 ).plusDatum( ( 7, 2, 5 ) ); neues Datum: 6.1.2003
        DLLEXPORT void plusDatum(int jahr, int monat, int tag);
        //! Addiert zum aktuellen Datum das bergebene Datum dazu und speichert
        //! das Ergebnis. \param format Eine Zeichenkette, die angibt in welcher
        //! Form das Datum in (datum) vorhanden ist. y=Jahr, m=Monat, d=Tag.
        //! \param datum Eine Zeichenkette, die das zu addierende Datum enthlt.
        //! Beispiel: ( 1.11.1995 ).plusDatum( "d.m.y", "5.2.7" ); neues
        //! Datum: 6.1.2003
        DLLEXPORT void plusDatum(const char* format, const char* datum);
        //! Addiert zum aktuellen Datum das bergebene Datum dazu und speichert
        //! das Ergebnis. \param format Eine Zeichenkette, die angibt in welcher
        //! Form das Datum in (datum) vorhanden ist. y=Jahr, m=Monat, d=Tag.
        //! \param datum Ein Text Zeichnung, welches das zu addierende Datum
        //! enthlt. Beispiel: ( 1.11.1995 ).plusDatum( "d.m.y", new Text(
        //! "5.2.7" ) ); neues Datum: 6.1.2003
        DLLEXPORT void plusDatum(const char* format, Text* datum);
        //! Addiert zum aktuellen Datum das bergebene Jahr dazu und speichert
        //! das Ergebnis. \param jahr Das zu addierende Jahr. Beispiel:
        //! ( 1.11.1995 ).plusJahr( 21 ); neues Datum: 1.11.2016
        DLLEXPORT void plusJahr(int jahr);
        //! Addiert zum aktuellen Datum den bergebenen Monat dazu und speichert
        //! das Ergebnis. \param monat Der zu addierende Monat. Beispiel:
        //! ( 1.11.1995 ).plusMonat( 13 ); neues Datum: 1.12.1996
        DLLEXPORT void plusMonat(int monat);
        //! Addiert zum aktuellen Datum den bergebenen Tag dazu und speichert
        //! das Ergebnis. \param tag Der zu addierende Tag. Beispiel: ( 1.1.2000
        //! ).plusTag( 32 ); neues Datum: 2.2.2000
        DLLEXPORT void plusTag(int tag);
        //! Zieht vom aktuellen Datum das bergebene Datum ab und speichert das
        //! Ergebnis. \param datum Das abzuzuehende Datum. Beispiel: ( 2.12.1996
        //! ).minusDatum( ( 1.1.1 ) ); neues Datum: 1.11.1995
        DLLEXPORT void minusDatum(Datum* datum);
        //! Zieht vom aktuellen Datum das bergebene Datum ab und speichert das
        //! Ergebnis. \param jahr Das abzuzuehende Jahr. \param monat Der
        //! abzuzuehende Monat. \param tag Der abzuzuehende Tag. Beispiel:
        //! ( 2.12.1996 ).minusDatum( 1, 1, 1 ); neues Datum: 1.11.1995
        DLLEXPORT void minusDatum(int jahr, int monat, int tag);
        //! Zieht vom aktuellen Datum das bergebene Datum ab und speichert das
        //! Ergebnis. \param format Eine Zeichenkette, die angibt in welcher
        //! Form das Datum in (datum) vorhanden ist. y=Jahr, m=Monat, d=Tag.
        //! \param datum Eine Zeichenkette, die das abzuziehende Datum enthlt.
        //! Beispiel: ( 2.12.1996 ).minusDatum( "d.m.y", "1.1.1" ); neues
        //! Datum: 1.11.1995
        DLLEXPORT void minusDatum(const char* format, const char* datum);
        //! Zieht vom aktuellen Datum das bergebene Datum ab und speichert das
        //! Ergebnis. \param format Eine Zeichenkette, die angibt in welcher
        //! Form das Datum in (datum) vorhanden ist. y=Jahr, m=Monat, d=Tag.
        //! \param datum Ein Text Zeichnung, welches das abzuziehende Datum
        //! enthlt. Beispiel: ( 2.12.1996 ).minusDatum( "d.m.y", new Text(
        //! "1.1.1" ) ); neues Datum: 1.11.1995
        DLLEXPORT void minusDatum(const char* format, Text* datum);
        //! Zieht vom aktuellen Datum das bergebene Jahr ab und speichert das
        //! Ergebnis. \param jahr Das abzuzuehende Jahr. Beispiel: ( 1.11.1996
        //! ).minusJahr( 1 ); neues Datum: 1.11.1995
        DLLEXPORT void minusJahr(int jahr);
        //! Zieht vom aktuellen Datum den bergebenen Monat ab und speichert das
        //! Ergebnis. \param monat Der abzuzuehende Monat. Beispiel: ( 1.12.1996
        //! ).minusMonat( 13 ); neues Datum: 1.11.1995
        DLLEXPORT void minusMonat(int monat);
        //! Zieht vom aktuellen Datum den bergebenen Tag ab und speichert das
        //! Ergebnis. \param monat Der abzuzuehende Tag. Beispiel: ( 5.2.2016
        //! ).minusMonat( 11 ); neues Datum: 25.1.2016
        DLLEXPORT void minusTag(int tag);

        //! gibt das Jahr zurck.
        DLLEXPORT int getJahr() const;
        //! gibt der Monat zurck.
        DLLEXPORT int getMonat() const;
        //! gibt der Tag zurck.
        DLLEXPORT int getTag() const;
        //! Gibt das Datum als Text Zeichnung zurck.
        //! \param format Eine Zeichenkette, die angibt in welcher Form das
        //! Datum zurckgegeben werden soll. y=Jahr, m=Monat, d=Tag. Beispiel:
        //! ( 1.11.1995 ).getDatum( "y-m-d" ); return: "1995-11-1"
        DLLEXPORT Text* getDatum(const char* format) const;
        //! Prft, ob das Datum gleich dem bergebenen Datum ist.
        //! \param datum das zu prfende Datum.
        //! \return (true), wenn das bergebene Datum dem gespeicherten
        //! entspricht. (false) sonnst.
        DLLEXPORT bool istGleich(Datum* datum) const;
        //! Prft, ob das Datum gleich dem bergebenen Datum ist.
        //! \param format Eine Zeichenkette, die angibt in welcher Form das
        //! Datum in (datum) vorhanden ist. y=Jahr, m=Monat, d=Tag. \param datum
        //! Eine Zeichenkette, die das zu berprfende Datum enthlt. \return
        //! (true), wenn das bergebene Datum dem gespeicherten entspricht.
        //! (false) sonnst. Beispiel: ( 1.11.1995 ).istGleich( "y-m-d",
        //! "1995-11-1" ); return: true
        DLLEXPORT bool istGleich(const char* format, const char* datum) const;
        //! Prft, ob das Datum gleich dem bergebenen Datum ist.
        //! \param format Eine Zeichenkette, die angibt in welcher Form das
        //! Datum in (datum) vorhanden ist. y=Jahr, m=Monat, d=Tag. \param datum
        //! Ein Text Zeichnung, welches das zu berprfende Datum enthlt.
        //! \return (true), wenn das bergebene Datum dem gespeicherten
        //! entspricht. (false) sonnst. Beispiel: ( 1.11.1995 ).istGleich(
        //! "y-m-d", new Text( "1995-11-1" ) ); return: true
        DLLEXPORT bool istGleich(const char* format, Text* datum) const;
        //! Prft, ob das Datum gleich dem bergebenen Datum ist.
        //! \param jahr Das Jahr des zu berprfenden Datums.
        //! \param monat Der Monat des zu berprfenden Datums.
        //! \param tag Der Tag des zu berprfenden Datums.
        //! \return (true), wenn das bergebene Datum dem gespeicherten
        //! entspricht. (false) sonnst. Beispiel: ( 1.11.1995 ).istGleich( 1995,
        //! 11, 1 ); return: true
        DLLEXPORT bool istGleich(int jahr, int monat, int tag) const;
        //! Prft, ob das Jahr gleich dem bergebenen Jahr ist.
        //! \param jahr Das zu prfende Jahr.
        //! \return (true), wenn das bergebene Jahr dem gespeicherten
        //! entspricht. (false) sonnst.
        DLLEXPORT bool jahrGleich(int jahr) const;
        //! Prft, ob der Monat gleich dem bergebenen Monat ist.
        //! \param monat Der zu prfende Monat.
        //! \return (true), wenn der bergebene Monat dem gespeicherten
        //! entspricht. (false) sonnst.
        DLLEXPORT bool monatGleich(int monat) const;
        //! Prft, ob der Tag gleich dem bergebenen Tag ist.
        //! \param tag Der zu prfende Tag.
        //! \return (true), wenn der bergebene Tag dem gespeicherten
        //! entspricht. (false) sonnst.
        DLLEXPORT bool tagGleich(int tag) const;
        //! Prft, ob das gespeicherte Datum kleiner als das bergebene ist.
        //! \param datum Das zu prfende Datum.
        //! \return (true), wenn das gespeicherte Datum vor dem bergebenen
        //! liegt. (false) sonnst. Beispiel: ( 1.11.1995 ).istKleiner(
        //! ( 23.1.2016 ) ); return true
        DLLEXPORT bool istKleiner(Datum* datum) const;
        //! Prft, ob das gespeicherte Datum kleiner als das bergebene ist.
        //! \param jahr Das Jahr des zu prfenden Datums.
        //! \param monat Der Monat des zu prfenden Datums.
        //! \param tag Der Tag des zu prfenden Datums.
        //! \return (true), wenn das gespeicherte Datum vor dem bergebenen
        //! liegt. (false) sonnst. Beispiel: ( 1.11.1995 ).istKleiner( 2016, 1,
        //! 23 ); return true
        DLLEXPORT bool istKleiner(int jahr, int monat, int tag) const;
        //! Prft, ob das gespeicherte Datum kleiner als das bergebene ist.
        //! \param format Eine Zeichenkette, die angibt in welcher Form das
        //! Datum in (datum) vorhanden ist. y=Jahr, m=Monat, d=Tag. \param datum
        //! Eine Zeichenkette, die das zu berprfende Datum enthlt. \return
        //! (true), wenn das gespeicherte Datum vor dem bergebenen liegt.
        //! (false) sonnst. Beispiel: ( 1.11.1995 ).istKleiner( "y, m, d",
        //! "2016, 1, 23" ); return true
        DLLEXPORT bool istKleiner(const char* format, const char* datum) const;
        //! Prft, ob das gespeicherte Datum kleiner als das bergebene ist.
        //! \param format Eine Zeichenkette, die angibt in welcher Form das
        //! Datum in (datum) vorhanden ist. y=Jahr, m=Monat, d=Tag. \param datum
        //! Ein Text Zeichnung, welches das zu berprfende Datum enthlt.
        //! \return (true), wenn das gespeicherte Datum vor dem bergebenen
        //! liegt. (false) sonnst. Beispiel: ( 1.11.1995 ).istKleiner( "y, m,
        //! d", new Text( "2016, 1, 23" ) ); return true
        DLLEXPORT bool istKleiner(const char* format, Text* datum) const;
        //! Prft, ob das gespeicherte Datum grer als das bergebene ist.
        //! \param datum Das zu prfende Datum.
        //! \return (true), wenn das gespeicherte Datum hinter dem bergebenen
        //! liegt. (false) sonnst. Beispiel: ( 1.11.1995 ).istGrer(
        //! ( 23.1.2016 ) ); return false
        DLLEXPORT bool istLater(Datum* datum) const;
        //! Prft, ob das gespeicherte Datum grer als das bergebene ist.
        //! \param jahr Das Jahr des zu prfenden Datums.
        //! \param monat Der Monat des zu prfenden Datums.
        //! \param tag Der Tag des zu prfenden Datums.
        //! \return (true), wenn das gespeicherte Datum hinter dem bergebenen
        //! liegt. (false) sonnst. Beispiel: ( 1.11.1995 ).istGrer( 2016, 1,
        //! 23 ); return false
        DLLEXPORT bool istLater(int jahr, int monat, int tag) const;
        //! Prft, ob das gespeicherte Datum grer als das bergebene ist.
        //! \param format Eine Zeichenkette, die angibt in welcher Form das
        //! Datum in (datum) vorhanden ist. y=Jahr, m=Monat, d=Tag. \param datum
        //! Eine Zeichenkette, die das zu berprfende Datum enthlt. \return
        //! (true), wenn das gespeicherte Datum hinter dem bergebenen liegt.
        //! (false) sonnst. Beispiel: ( 1.11.1995 ).istGrer( "y, m, d", "2016,
        //! 1, 23" ); return false
        DLLEXPORT bool istLater(const char* format, const char* datum) const;
        //! Prft, ob das gespeicherte Datum grer als das bergebene ist.
        //! \param format Eine Zeichenkette, die angibt in welcher Form das
        //! Datum in (datum) vorhanden ist. y=Jahr, m=Monat, d=Tag. \param datum
        //! Ein Text Zeichnung, welches das zu berprfende Datum enthlt.
        //! \return (true), wenn das gespeicherte Datum hinter dem bergebenen
        //! liegt. (false) sonnst. Beispiel: ( 1.11.1995 ).istGrer( "y, m, d",
        //! new Text( "2016, 1, 23" ) ) ); return true
        DLLEXPORT bool istLater(const char* format, Text* datum) const;
    };

    //! Diese Klasse vereint die beiden Klassen Datum und Uhrzeit und speichert
    //! somit einen Zeitstempel mit Jahr, Monat, Tag, Stunde, Minute und Sekunde
    class Zeit : public virtual ReferenceCounter
    {
    private:
        Datum* datum;
        Uhrzeit* uhrzeit;

    public:
        //! Erzeugt eine neue Zeit mit den Standartwerten 0.0.0 0:0:0.
        DLLEXPORT Zeit();
        //! Erzeugt eine neue Zeit mit den vergangenen sekunden seit 1970.
        DLLEXPORT Zeit(__int64 timestamp);
        //! Lscht das aktuelle Zeichnung.
        DLLEXPORT ~Zeit();
        //! ndert die gespeicherte Zeit durch kopieren der Werte aus (zeit).
        //! \param zeit Die neue Zeit.
        DLLEXPORT void setZeit(Zeit* zeit);
        //! ndert die gespeicherte Zeit.
        //! \param jahr Das neue Jahr.
        //! \param monat Der neue Monat.
        //! \param tag Det neue Tag.
        //! \param stunde Die neue Stunde.
        //! \param minute Die neue Minute.
        //! \param sekunde Die neue Sekunde.
        DLLEXPORT void setZeit(
            int jahr, int monat, int tag, int stunde, int minute, int sekunde);
        //! ndert die gespeicherte Zeit.
        //! \param format Eine Zeichenkette, die angibt in welcher Form die neue
        //! Zeit in (zeit) vorhanden ist. y=Jahr, m=Monat, d=Tag, h=Stunde,
        //! i=Minute, s=Sekunde. \param zeit Eine Zeichenkette, die die neue
        //! Zeit enthlt. Beispiel: setZeit( "y-m-d h:i:s", "2016-1-25 21:59:30"
        //! );
        DLLEXPORT void setZeit(const char* format, const char* zeit);
        //! ndert die gespeicherte Zeit.
        //! \param format Eine Zeichenkette, die angibt in welcher Form die neue
        //! Zeit in (zeit) vorhanden ist. y=Jahr, m=Monat, d=Tag, h=Stunde,
        //! i=Minute, s=Sekunde. \param zeit Ein Text Zeichnung, welches die
        //! neue Zeit enthlt. Beispiel: setZeit( "y-m-d h:i:s", new Text(
        //! "2016-1-25 21:59:30" ) );
        DLLEXPORT void setZeit(const char* format, Text* zeit);
        //! ndert das gespeicherte Jahr.
        //! \param jahr Das neue Jahr.
        DLLEXPORT void setJahr(int jahr);
        //! ndert den gespeicherten Monat.
        //! \param monat Der neue Monat.
        DLLEXPORT void setMonat(int monat);
        //! ndert den gespeicherten Tag.
        //! \param tag Der neue Tag.
        DLLEXPORT void setTag(int tag);
        //! ndert die gespeicherte Stunde.
        //! \param stunde Die neue Stunde.
        DLLEXPORT void setStunde(int stunde);
        //! ndert die gespeicherte Minute.
        //! \param minute Die neue Minute.
        DLLEXPORT void setMinute(int minute);
        //! ndert die gespeicherte Sekunde.
        //! \param sekunde Die neue Sekunde.
        DLLEXPORT void setSekunde(int sekunde);
        //! Addiert die bergebene Zeit und speichert das Ergebnis.
        //! \param zeit Die zu Addierende Zeit.
        DLLEXPORT void plusZeit(Zeit* zeit);
        //! Addiert die bergebene Zeit und speichert das Ergebnis.
        //! \param jahr Das zu addierende Jahr.
        //! \param monat Der zu addierende Monat.
        //! \param tag Der zu addierende Tag.
        //! \param stunde Die zu addierende Stunde.
        //! \param minute Die zu addierende Minute.
        //! \param sekunde Die zu addierende Sekunde.
        DLLEXPORT void plusZeit(
            int jahr, int monat, int tag, int stunde, int minute, int sekunde);
        //! Addiert die bergebene Zeit und speichert das Ergebnis.
        //! \param format Eine Zeichenkette, die angibt in welcher Form die neue
        //! Zeit in (zeit) vorhanden ist. y=Jahr, m=Monat, d=Tag, h=Stunde,
        //! i=Minute, s=Sekunde. \param zeit Eine Zeichenkette, die die zu
        //! addierende Zeit enthlt.
        DLLEXPORT void plusZeit(const char* format, const char* zeit);
        //! Addiert die bergebene Zeit und speichert das Ergebnis.
        //! \param format Eine Zeichenkette, die angibt in welcher Form die neue
        //! Zeit in (zeit) vorhanden ist. y=Jahr, m=Monat, d=Tag, h=Stunde,
        //! i=Minute, s=Sekunde. \param zeit Ein Text Zeichnung, welches die zu
        //! addierende Zeit enthlt.
        DLLEXPORT void plusZeit(const char* format, Text* zeit);
        //! Addiert die bergebene Zeit und speichert das Ergebnis.
        //! \param jahr Das zu addierende Jahr.
        DLLEXPORT void plusJahr(int jahr);
        //! Addiert die bergebene Zeit und speichert das Ergebnis.
        //! \param monat Der zu addierende Monat.
        DLLEXPORT void plusMonat(int monat);
        //! Addiert die bergebene Zeit und speichert das Ergebnis.
        //! \param tag Der zu addierende Tag.
        DLLEXPORT void plusTag(int tag);
        //! Addiert die bergebene Zeit und speichert das Ergebnis.
        //! \param stunde Die zu addierende Stunde.
        DLLEXPORT void plusStunde(__int64 stunde);
        //! Addiert die bergebene Zeit und speichert das Ergebnis.
        //! \param minute Die zu addierende Minute.
        DLLEXPORT void plusMinute(__int64 minute);
        //! Addiert die bergebene Zeit und speichert das Ergebnis.
        //! \param sekunde Die zu addierende Sekunde.
        DLLEXPORT void plusSekunde(__int64 sekunde);
        //! Zieht die bergebene Zeit ab und speichert das Ergebnis.
        //! \param zeit Die abzuziehende Zeit.
        DLLEXPORT void minusZeit(Zeit* zeit);
        //! Zieht die bergebene Zeit ab und speichert das Ergebnis.
        //! \param jahr Das abzuziehende Jahr.
        //! \param monat Der abzuziehende Monat.
        //! \param tag Der abzuziehende Tag.
        //! \param stunde Die abzuziehende Stunde.
        //! \param minute Die abzuziehende Minute.
        //! \param sekunde Die abzuziehende Sekunde.
        DLLEXPORT void minusZeit(
            int jahr, int monat, int tag, int stunde, int minute, int sekunde);
        //! Zieht die bergebene Zeit ab und speichert das Ergebnis.
        //! \param format Eine Zeichenkette, die angibt in welcher Form die neue
        //! Zeit in (zeit) vorhanden ist. y=Jahr, m=Monat, d=Tag, h=Stunde,
        //! i=Minute, s=Sekunde. \param zeit Eine Zeichenkette, die die
        //! abzuziehende Zeit enthlt.
        DLLEXPORT void minusZeit(const char* format, const char* zeit);
        //! Zieht die bergebene Zeit ab und speichert das Ergebnis.
        //! \param format Eine Zeichenkette, die angibt in welcher Form die neue
        //! Zeit in (zeit) vorhanden ist. y=Jahr, m=Monat, d=Tag, h=Stunde,
        //! i=Minute, s=Sekunde. \param zeit Ein Text Zeichnung, welches die
        //! abzuziehende Zeit enthlt.
        DLLEXPORT void minusZeit(const char* format, Text* zeit);
        //! Zieht die bergebene Zeit ab und speichert das Ergebnis.
        //! \param jahr Das abzuziehende Jahr.
        DLLEXPORT void minusJahr(int jahr);
        //! Zieht die bergebene Zeit ab und speichert das Ergebnis.
        //! \param monat Der abzuziehende Monat.
        DLLEXPORT void minusMonat(int monat);
        //! Zieht die bergebene Zeit ab und speichert das Ergebnis.
        //! \param tag Der abzuziehende Tag.
        DLLEXPORT void minusTag(int tag);
        //! Zieht die bergebene Zeit ab und speichert das Ergebnis.
        //! \param stunde Die abzuziehende Stunde.
        DLLEXPORT void minusStunde(__int64 stunde);
        //! Zieht die bergebene Zeit ab und speichert das Ergebnis.
        //! \param minute Die abzuziehende Minute.
        DLLEXPORT void minusMinute(__int64 minute);
        //! Zieht die bergebene Zeit ab und speichert das Ergebnis.
        //! \param sekunde Die abzuziehende Sekunde.
        DLLEXPORT void minusSekunde(__int64 sekunde);
        //! Gibt die gespeicherte Zeit als Text zurck.
        //! \param format Eine Zeichenkette, die angibt in welcher Form die neue
        //! Zeit zurckgegeben werden soll. y=Jahr, m=Monat, d=Tag, h=Stunde,
        //! i=Minute, s=Sekunde.
        DLLEXPORT Text* getZeit(const char* format) const;
        //! berprft, ob die Zeit gleich der bergebenen Zeit ist.
        //! \param zeit die zu berprfende Zeit.
        //! \return (true), wenn die Zeiten gleich sind. (false) sonst.
        DLLEXPORT bool istGleich(Zeit* zeit) const;
        //! berprft, ob die Zeit gleich der bergebenen Zeit ist.
        //! \param format Eine Zeichenkette, die angibt in welcher Form die neue
        //! Zeit in (zeit) vorhanden ist. y=Jahr, m=Monat, d=Tag, h=Stunde,
        //! i=Minute, s=Sekunde. \param zeit Eine Zeichenkette, die die zu
        //! berprfende Zeit enthlt. \return (true), wenn die Zeiten gleich
        //! sind. (false) sonst.
        DLLEXPORT bool istGleich(const char* format, const char* zeit) const;
        //! berprft, ob die Zeit gleich der bergebenen Zeit ist.
        //! \param format Eine Zeichenkette, die angibt in welcher Form die neue
        //! Zeit in (zeit) vorhanden ist. y=Jahr, m=Monat, d=Tag, h=Stunde,
        //! i=Minute, s=Sekunde. \param zeit Eine Text Zeichnung, welches die zu
        //! berprfende Zeit enthlt. \return (true), wenn die Zeiten gleich
        //! sind. (false) sonst.
        DLLEXPORT bool istGleich(const char* format, Text* zeit) const;
        //! berprft, ob die Zeit gleich der bergebenen Zeit ist.
        //! \param jahr Das zu berprfende Jahr.
        //! \param monat Der zu berprfende Monat.
        //! \param tag Der zu berprfende Tag.
        //! \param stunde Die zu berprfende Stunde.
        //! \param minute Die zu berprfende Minute.
        //! \param sekunde Die zu berprfende Sekunde.
        //! \return (true), wenn die Zeiten gleich sind. (false) sonst.
        DLLEXPORT bool istGleich(int jahr,
            int monat,
            int tag,
            int stunde,
            int minute,
            int sekunde) const;
        //! Gibt das Zeichnung zurck, in dem das Datum gespeichert wird mit
        //! erhhtem Reference Counter.
        DLLEXPORT Datum* getDatum() const;
        //! Gibt das Zeichnung zurck, in dem das Datum gespeichert wird ohne
        //! erhhten Reference Counter.
        DLLEXPORT Datum* zDatum() const;
        //! Gibt das Zeichnung zurck, in dem die Uhrzeit gespeichert wird mit
        //! erhhtem Reference Counter.
        DLLEXPORT Uhrzeit* getUhrzeit() const;
        //! Gibt das Zeichnung zurck, in dem die uhrzeit gespeichert wird ohne
        //! erhhten Reference Counter.
        DLLEXPORT Uhrzeit* zUhrzeit() const;
        //! berprft, ob die gespeicherte Zeit kleiner als die bergebene Zeit
        //! ist. \param zeit Die zu berprfende Zeit. \return (true) wenn die
        //! gespeicherte Zeit vor der bergebenen Zeit liegt. (false) sonnst.
        DLLEXPORT bool istKleiner(Zeit* zeit) const;
        //! berprft, ob die gespeicherte Zeit kleiner als die bergebene Zeit
        //! ist. \param jahr Das zu berprfende Jahr. \param monat Der zu
        //! berprfende Monat. \param tag Der zu berprfende Tag. \param
        //! stunde Die zu berprfende Stunde. \param minute Die zu berprfende
        //! Minute. \param sekunde Die zu berprfende Sekunde. \return (true)
        //! wenn die gespeicherte Zeit vor der bergebenen Zeit liegt. (false)
        //! sonnst.
        DLLEXPORT bool istKleiner(int jahr,
            int monat,
            int tag,
            int stunde,
            int minute,
            int sekunde) const;
        //! berprft, ob die gespeicherte Zeit kleiner als die bergebene Zeit
        //! ist. \param format Eine Zeichenkette, die angibt in welcher Form die
        //! neue Zeit in (zeit) vorhanden ist. y=Jahr, m=Monat, d=Tag, h=Stunde,
        //! i=Minute, s=Sekunde. \param zeit Eine Zeichenkette, die die zu
        //! berprfende Zeit enthlt. \return (true) wenn die gespeicherte Zeit
        //! vor der bergebenen Zeit liegt. (false) sonnst.
        DLLEXPORT bool istKleiner(const char* format, const char* zeit) const;
        //! berprft, ob die gespeicherte Zeit kleiner als die bergebene Zeit
        //! ist. \param format Eine Zeichenkette, die angibt in welcher Form die
        //! neue Zeit in (zeit) vorhanden ist. y=Jahr, m=Monat, d=Tag, h=Stunde,
        //! i=Minute, s=Sekunde. \param zeit Eine Text Zeichnung, welches die zu
        //! berprfende Zeit enthlt. \return (true) wenn die gespeicherte Zeit
        //! vor der bergebenen Zeit liegt. (false) sonnst.
        DLLEXPORT bool istKleiner(const char* format, Text* zeit) const;
        //! berprft, ob die gespeicherte Zeit grer als die bergebene Zeit
        //! ist. \param zeit Die zu berprfende Zeit. \return (true) wenn die
        //! gespeicherte Zeit nach der bergebenen Zeit liegt. (false) sonnst.
        DLLEXPORT bool istLater(Zeit* zeit) const;
        //! berprft, ob die gespeicherte Zeit grer als die bergebene Zeit
        //! ist. \param jahr Das zu berprfende Jahr. \param monat Der zu
        //! berprfende Monat. \param tag Der zu berprfende Tag. \param
        //! stunde Die zu berprfende Stunde. \param minute Die zu berprfende
        //! Minute. \param sekunde Die zu berprfende Sekunde. \return (true)
        //! wenn die gespeicherte Zeit nach der bergebenen Zeit liegt. (false)
        //! sonnst.
        DLLEXPORT bool istLater(int jahr,
            int monat,
            int tag,
            int stunde,
            int minute,
            int sekunde) const;
        //! berprft, ob die gespeicherte Zeit grer als die bergebene Zeit
        //! ist. \param format Eine Zeichenkette, die angibt in welcher Form die
        //! neue Zeit in (zeit) vorhanden ist. y=Jahr, m=Monat, d=Tag, h=Stunde,
        //! i=Minute, s=Sekunde. \param zeit Eine Zeichenkette, die die zu
        //! berprfende Zeit enthlt. \return (true) wenn die gespeicherte Zeit
        //! nach der bergebenen Zeit liegt. (false) sonnst.
        DLLEXPORT bool istLater(const char* format, const char* zeit) const;
        //! berprft, ob die gespeicherte Zeit grer als die bergebene Zeit
        //! ist. \param format Eine Zeichenkette, die angibt in welcher Form die
        //! neue Zeit in (zeit) vorhanden ist. y=Jahr, m=Monat, d=Tag, h=Stunde,
        //! i=Minute, s=Sekunde. \param zeit Eine Text Zeichnung, welches die zu
        //! berprfende Zeit enthlt. \return (true) wenn die gespeicherte Zeit
        //! nach der bergebenen Zeit liegt. (false) sonnst.
        DLLEXPORT bool istLater(const char* format, Text* zeit) const;
    };

    //! Diese Klasse kann messen, wie viel Zeit zwischen zwei Zeitpunkten
    //! verstrichen ist
    class ZeitMesser : public virtual ReferenceCounter
    {
    private:
        double start;
        double ende;
        double messung;

    public:
        //! Erzeugt ein neues ZeitMesser Zeichnung
        DLLEXPORT ZeitMesser();
        //! Lscht das aktuelle Zeichnung
        DLLEXPORT ~ZeitMesser();
        //! legt den Startpunkt des zeitstoppens fest
        DLLEXPORT void messungStart();
        //! legt des Endpunkt der Zeitmessung fest
        DLLEXPORT void messungEnde();
        //! gibt den Abstand zwischen start und ende der Messung in Sekunden
        //! zurck
        DLLEXPORT double getSekunden() const;
        //! gibt den Abstand zwischen start und ende der Messung in Minuten
        //! zurck
        DLLEXPORT double getMinuten() const;
        //! gibt den Abstand zwischen start und ende der Messung in Stunden
        //! zurck
        DLLEXPORT double getStunden() const;
        //! gibt true zurck, wenn aktuell eine MEssung luft
        DLLEXPORT bool isMeasuring() const;
    };

    //! gibt die aktuelle Uhrzeit zurck.
    DLLEXPORT Uhrzeit* getUhrzeit();
    //! gibt das aktuelle Datum zurck.
    DLLEXPORT Datum* getDatum();
    //! gibt die aktuelle Zeit( Datum und Uhrzeit ) zurck.
    DLLEXPORT Zeit* getZeit();
    //! prft, ob jahr ein Schaltjahr ist.
    //! \param jahr Das zu berprfende Jahr.
    //! \return (true), wenn das bergebene jahr ein Schaltjahr ist. (false)
    //! sonst.
    DLLEXPORT bool istSchaltjahr(int jahr);
} // namespace Framework

#endif